package com.css.simulation.resource.server.app.impl;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.vo.project.BaseSceneKeysNumsVo;
import api.common.pojo.vo.project.SceneComputerECRateVo;
import api.common.pojo.vo.scene.CoveragePrefixStatusVO;
import api.common.util.CollectionUtil;
import api.common.util.StringUtil;
import com.alibaba.fastjson.JSONObject;
import com.css.simulation.resource.server.app.service.SceneComputerRateService;
import com.css.simulation.resource.server.infra.db.entity.SceneComputerRateEntity;
import com.css.simulation.resource.server.infra.db.mysql.mapper.SceneComputerRateMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author zhoutianfu
 * @update 2025/3/2 13:23
 * @since 2025/3/2 13:23
 **/
@Service
@Slf4j
@RequiredArgsConstructor
public class SceneComputerRateServiceImpl implements SceneComputerRateService {

    private final SceneComputerRateMapper sceneComputerRateMapper;

    @Override
    public void saveSceneComputerRate(SceneComputerRateEntity params) {
        sceneComputerRateMapper.saveSceneComputerRate(params);
    }

    @Override
    public void deleteByIdAndType(String computerRateId, String computerType) {
        sceneComputerRateMapper.deleteByIdAndType(computerRateId, computerType);
    }

    @Override
    public List<SceneComputerECRateVo> selectProjectComputerRates(String projectId, List<String> computerTypeList) {

        List<SceneComputerRateEntity> sceneComputerRateEntities = sceneComputerRateMapper.loadSceneComputerRateByTaskIdAndType(projectId, computerTypeList);
        Map<String, SceneComputerECRateVo> sceneComputerECRateVoMap = new HashMap<>();
        if (!CollectionUtil.isEmpty(sceneComputerRateEntities)) {
            for (SceneComputerRateEntity sceneComputerRateEntity : sceneComputerRateEntities) {
                String computerResult = sceneComputerRateEntity.getComputerResult();
                if (StringUtil.isEmpty(computerResult)) {
                    continue;
                }
                List<JSONObject> jsonObjects = JSONObject.parseArray(computerResult, JSONObject.class);
                for (JSONObject jsonObject : jsonObjects) {
                    String sceneName = (String) jsonObject.get("场景库名称");
                    SceneComputerECRateVo orDefault = sceneComputerECRateVoMap.getOrDefault(sceneName, new SceneComputerECRateVo());
                    orDefault.setSceneBaseName(sceneName);
                    if (DictConstants.COVERAGE.equals(sceneComputerRateEntity.getComputerType())) {
                        Object coverageRate = jsonObject.getOrDefault("覆盖率", 0.0);
                        BigDecimal bigDecimal = new BigDecimal(coverageRate.toString());
                        orDefault.setCoverageRate(bigDecimal.doubleValue());
                    } else if (DictConstants.EXPOSURE_RATE.equals(sceneComputerRateEntity.getComputerType())) {
                        Object exposureRate = jsonObject.getOrDefault("暴露率", 0.0);
                        BigDecimal bigDecimal = new BigDecimal(exposureRate.toString());
                        orDefault.setExposureRate(bigDecimal.doubleValue());
                    } else {
                        continue;
                    }
                    sceneComputerECRateVoMap.put(sceneName, orDefault);
                }
            }
        }
        return new ArrayList<>(sceneComputerECRateVoMap.values());
    }

    @Override
    public SceneComputerRateEntity loadSceneCoveragePrefixKeyNums() {
        return sceneComputerRateMapper.loadSceneCoveragePrefixKeyNums(DictConstants.COVERAGE_PREFIX);
    }

    @Override
    public CoveragePrefixStatusVO loadConvergePrefixStatus() {
        SceneComputerRateEntity sceneComputerRateEntity = this.loadSceneCoveragePrefixKeyNums();
        CoveragePrefixStatusVO coveragePrefixStatusVO = new CoveragePrefixStatusVO();
        if (sceneComputerRateEntity != null) {
            if (!StringUtil.isEmpty(sceneComputerRateEntity.getComputerResult())) {
                coveragePrefixStatusVO.setStatusCode(DictConstants.PROJECT_COMPLETED);
                coveragePrefixStatusVO.setStatusMessage("执行完成");
                String computerResult = sceneComputerRateEntity.getComputerResult();
                log.info("ConvergePrefix computerResult={}", computerResult);
                List<JSONObject> jsonObjects = JSONObject.parseArray(computerResult, JSONObject.class);
                List<BaseSceneKeysNumsVo> baseSceneKeysNumsVos = new ArrayList<>(jsonObjects.size());
                for (JSONObject jsonObject : jsonObjects) {
                    String sceneName = (String) jsonObject.get("场景库名称");
                    Integer keyNums = (Integer) jsonObject.get("关键场景个数");
                    BaseSceneKeysNumsVo baseSceneKeysNumsVo = new BaseSceneKeysNumsVo(sceneName, keyNums);
                    baseSceneKeysNumsVos.add(baseSceneKeysNumsVo);
                }
                coveragePrefixStatusVO.setBaseSceneKeysNumsVos(baseSceneKeysNumsVos);
            } else {
                coveragePrefixStatusVO.setStatusCode(DictConstants.PROJECT_RUNNING);
                coveragePrefixStatusVO.setStatusMessage(DictConstants.PROJECT_RUNNING_NAME);
            }
        } else {
            coveragePrefixStatusVO.setStatusCode(DictConstants.PROJECT_WAITING);
            coveragePrefixStatusVO.setStatusMessage("未执行");
        }
        return coveragePrefixStatusVO;
    }
}
